@@ -8,7 +8,7 @@ from .stream import Stream
 
 class Client:
 
-    def __init__(self, system_url='https://api.vyze.io/system/', stream_url='wss://api.vyze.io/stream', app_url='https://api.vyze.io/app/'):
+    def __init__(self, system_url='https://api.vyze.io/system/', stream_url='wss://api.vyze.io/stream', app_url='https://api.vyze.io/app/', timeout=300):
         self.__app_url = app_url
         self.__stream_url = stream_url
         self.__system_url = system_url
@@ -27,6 +27,8 @@ class Client:
         self.__targets_cache = {}
         self.__origins_cache = {}
 
+        self.__timeout = timeout
+
     def login(self, username, password):
         resp = requests.post(f'{self.__app_url}user/login', json={
             'username': username,
@@ -126,8 +128,20 @@ class Client:
     def get_data(self, id: str) -> bytes:
         return self.__get(f'{self.__system_url}objects/{self.resolve(id)}/data', is_json=False)
 
-    def set_data(self, id: str, data: bytes):
-        self.__post(f'{self.__system_url}objects/{self.resolve(id)}/data', data, is_json=False)
+    def set_data(self, id: str, data: bytes, chunks=None, update=None):
+        if not chunks:
+            return self.__post(f'{self.__system_url}objects/{self.resolve(id)}/data', data, is_json=False) is not False
+        else:
+            offset = 0
+            while True:
+                update(offset, len(data))
+                data_chunk = data[offset:min(offset + chunks, len(data))]
+                if self.__post(f'{self.__system_url}objects/{self.resolve(id)}/data?a=1', data_chunk, is_json=False) is False:
+                    return False
+                offset += chunks
+                if offset >= len(data):
+                    update(len(data), len(data))
+                    return True
 
     def get_abstracts(self, id: str, include_self=False, include_direct=True, include_transitive=False):
         return self.__get_hierarchy(self.resolve(id), include_self, include_direct, include_transitive, 'abstracts', self.__abstract_cache)
@@ -244,7 +258,7 @@ class Client:
         return rel
 
     def __get(self, url, is_json=True, **kwargs):
-        resp = requests.get(url, headers=self.__get_headers(), **kwargs)
+        resp = requests.get(url, headers=self.__get_headers(), timeout=self.__timeout, **kwargs)
         if resp.status_code == 200:
             return resp.json() if is_json else resp.content
         elif resp.status_code == 403:
@@ -254,21 +268,24 @@ class Client:
 
     def __post(self, url, data, is_json=True, **kwargs):
         if is_json:
-            resp = requests.post(url, json=data, headers=self.__get_headers(), **kwargs)
+            resp = requests.post(url, json=data, headers=self.__get_headers(), timeout=self.__timeout, **kwargs)
         else:
-            resp = requests.post(url, data=data, headers=self.__get_headers(), **kwargs)
+            resp = requests.post(url, data=data, headers=self.__get_headers(), timeout=self.__timeout, **kwargs)
         if resp.status_code == 200:
             return resp.json() if is_json else resp.content
         elif resp.status_code == 403:
             if self.__request_permission(resp.text):
                 return self.__post(url, data=data, is_json=is_json, **kwargs)
-        return None
+            else:
+                return False
+        else:
+            return False
 
     def __put(self, url, data, is_json=True, **kwargs):
         if is_json:
-            resp = requests.post(url, json=data, headers=self.__get_headers(), **kwargs)
+            resp = requests.post(url, json=data, headers=self.__get_headers(), timeout=self.__timeout, **kwargs)
         else:
-            resp = requests.post(url, data=data, headers=self.__get_headers(), **kwargs)
+            resp = requests.post(url, data=data, headers=self.__get_headers(), timeout=self.__timeout, **kwargs)
         if resp.status_code == 200:
             return resp.json() if is_json else resp.content
         elif resp.status_code == 403:
@@ -277,7 +294,7 @@ class Client:
         return None
 
     def __delete(self, url, is_json=True, **kwargs):
-        resp = requests.delete(url, headers=self.__get_headers(), **kwargs)
+        resp = requests.delete(url, headers=self.__get_headers(), timeout=self.__timeout, **kwargs)
         if resp.status_code == 200:
             return resp.json() if is_json else resp.content
         elif resp.status_code == 403:
@@ -289,7 +306,7 @@ class Client:
         resp = requests.post(f'{self.__app_url}user/refresh', json={
             'userId': self.__user_id,
             'refreshToken': self.__refresh_token,
-        })
+        }, timeout=self.__timeout)
 
         if resp.status_code != 200:
             return False
